// Modelo: "Samba: DSGE-VAR Model with a Bayesian Approach".
// Autores: Max Resende and Guilherme Vale Moura (Janeiro 2015) 
//**************************************************************************
// Variveis do modelo                                                    //*

var co crot c no nrot n q wr k u fii qi i x m rk mc pi bystar nxy r gy syhat 
bby g y yva piva pibar sgbar zc zn zi zfiistar zfii a zr zg  mstar pistar rstar;
//**************************************************************************
// Descrio das variveis do modelo                                      //*

   // co          consumo timo das famlias 
   // crot        consumo da regra de ouro das famlias  
   // c           consumo agregado 
   // no          oferta de trabalho tima das famlias
   // nrot        oferta de trabalho a regra de ouro das famlias    
   // n           oferta ou demanda de trabalho agegrada
   // q           taxa de cmbio real - condio paridade descoberta de juros
   // wr          salrio nominal
   // k           estoque de capital
   // u           taxa de utilizao do capital
   // fii         prmio de risco do pas
   // qi          vem da equao de Euler do capital
   // i           investimento
   // x           exportao
   // m           importaes 
   // rk          utilizao do capital
   // mc          custo marginal real
   // pi          inflao
   // bystar      ativos externos lquidos (como proporo do PIB)
   // r           taxa de juros
   // gy          gastos do governo (como proporo do PIB)
   // syhat       supervit primrio
   // bby         dvida do governo (como proporo do PIB)
   // g           gastos do governo
   // y           PIB
   // yva         valor adicionado
   // piva        valor adicionado da inflao (deflator do PIB)
   // pibar       meta de inflao
   // sgbar       meta fiscal
   // zc          preferncia das famlias
   // zn          choque da oferta de trabalho
   // zi          choque do investimento
   // zfiistar    averso ao risco do investidor externo
   // zfii        choque no prmio de risco do pas
   // a           tecnologia
   // zr          choque de poltica monetria
   // zg          choque de poltica fiscal
   // mstar       importaes do resto do mundo
   // pistar      inflao do resto do mundo
   // rstar       taxa de juros do resto do mundo
//**************************************************************************
//Variveis exgenas                                                      //*

varexo  c_ i_ fiistar_ a_ rstar_ r_ pistar_;

// fiscal_ interest_ gbar_ g_ mstar_ fii_ n_ pibar_;
                                         //*
//**************************************************************************
//Parmetros                                                              //*

parameters 
                                                                     
beta alpha sigma pessi kappa h omegabarb tet nuu kuu vi del dela dels sc si 
sg sm sx sva sd omegabarc omegabarn iok fiistar rstarC bystarC rC byC gamar 
gamapi gamay gamag gamas gamab rhoa rhoc rhoi rhon rhoq rhor rhog rhosbar 
rhopi rhofii rhofiistar rhomstar rhorstar rhopistar;
//coffispol;
//**************************************************************************
//Parmetros de gosto e tecnologia                                        //*
                                                                          //*
beta = 0.98;        //fator de desconto
alpha = 0.4;        //percentagem do capital na funo de produo domstica
//sigma = 0.823;      //inverso da elasticidade intertemporal de substituio
//pessi=0.04;         //elasticidade do risco com respeito a ativos externos lquidos
//kappa = 1.014;      //elasticidade das exportaes com respeito a taxa de cmbio
//h = 0.218;          //parmetro de persistncia do hbito
//omegabarb=0.239;    //percentagem das firmas da regra de ouro no grupo das firmas que ajustam seus preos
//tet=0.906;          //percentagem das firmas que no ajustam seus preo (parmetro de Calvo)
//nuu=0.338;          //inverso da elasticidade da oferta de trabalho
//kuu=1.406;          //elasticidade de substituio entre bens domsticos e importados
//vi=0.672;           //elasticidade do prmio de risco
del=0.025;          //taxa de depreciao do capital
dela=1;             //curvatura da funo custo de capacidade de utilizao
//dels=3.997;         //segunda derivada do custo de ajustamento do investimento
//**************************************************************************
//Variveis e relaes no estado estacionrio                             //*
                                                                          //*
sc=0.555;           //percentual do consumo no PIB
si=0.147;           //percentual do investimento no PIB
sg=0.176;           //percentual dos gastos do governo no PIB
sm=0.109;           //percentual da importao no PIB
sx=0.121;           //pecentual das exportaes no PIB
sva=0.891;          //percentual do valor adicionado no PIB
sd=0.891;           //percentual de insumo domstico no PIB
omegabarc=0.4;      //percentual do consumo na regra de ouro das famlias
omegabarn=0.8;      //percentual de emprego na regra de ouro das famlias
iok=0.03;           //razo capital investimento
fiistar=1.016;      //prmio de risco do pas
rstarC=1.009;       //taxa de juros nominal do resto do mundo bruta
bystarC=0.033;      //razo ativos externos lquidos-valor adicionado
rC=1.042;           //taxa de juros nominal domstica bruta
byC=1.028;          //razo dvida do governo-valor adicionado
//**************************************************************************
//Parmetros de poltica                                                  //*
                                                                          //*
//gamar=0.682;        //suavizao da taxa de juros na regra de Taylor
//gamapi=1.523;       //coeficiente da inflao na regra de Taylor
//gamay=0.839;        //coeficiente do hiato do produto na regra de taylor
//gamag=0.695;        //suavizao dos gastos do governo na regra fiscal
//gamas=0.465;        //coeficiente do supervit primrio na regra fiscal
//gamab=0.17;         //coeficiente da dvida do governo na regra fiscal
//**************************************************************************
//Processos auto-regressivos                                              //*
                                                                          //*
//rhoa = 0.985;       //persistncia do choque tecnolgico
//rhoc = 0.7;         //persistncia do choque do consumo
//rhoi = 0.28;        //persistncia do choque do investimento
//rhon = 0.74;        //persistncia do choque oferta de trabalho
rhoq = 0.0;         //persistncia do choque da condio paridade descoberta de juros
rhor = 0.0;         //persistncia do choque de poltica monetria
rhog = 0.0;         //persistncia do choque de poltica fiscal
//rhosbar = 0.915;    //persistncia do choque da meta de supervit primrio
//rhopi = 0.937;      //persistncia do choque da meta de inflao
//rhofii = 0.867;     //persistncia do choque do prmio de risco pas
//rhofiistar = 0.64;  //persistncia do choque de averso ao risco do investidor externo
//rhomstar = 0.257;   //persistncia do choque de importaes do resto do mundo
//rhorstar = 0.848;   //persistncia do choque de taxa de juros do resto do mundo
//rhopistar = -0.139; //persistncia do choque de inflao do resto mundo
//**************************************************************************
// Definio do Parmetro de Poltica Fiscal                             //*
//coffispol = 1;                                                           //*
//**************************************************************************

model(linear);
//**************************************************************************
// Discricionariedade dos gastos do governo                              //*
                                                                         //*
//g_ = coffispol*fiscal_;                                                  //*
//**************************************************************************
// Regra de Poltica                                                     //*

// Taylor rule:
r = gamar*r(-1)+(1-gamar)*(gamapi*(pi(+1)-pibar(+1))+pibar+gamay*yva)+zr;

// Fiscal policy rule:
gy = gamag*gy(-1)+(1-gamag)*(gamas*syhat(-1)- gamab*bby(-1)) + zg;           
//**************************************************************************
// Consumption of optimizing households:
co = (1/(1+h))*co(+1)+(h/(1+h))*co(-1)-sigma^(-1)*((1-h)/(1+h))*(r-pi(+1))+sigma^(-1)*((1-h)/(1+h))*(1-rhoc)*zc;

// Consumption of rule-of-thumb households: 
crot= wr+nrot;

// Aggregate consumption:
c = (1-omegabarc)*co + omegabarc*crot;

// Labor supply of optimizing households:
no = nuu^(-1)*(wr-(sigma/(1-h))*(co-h*co(-1))-zn);

// Labor supply of rule-of-thumb households:
nrot = nuu^(-1)*(wr-(sigma/(1-h))*(crot-h*crot(-1))-zn);

// Aggregate labor supply:
n = (1-omegabarn)*no + omegabarn*nrot;

// UIP condition:
q = q(+1)-((r-pi(+1))-(rstar+fii-pistar(+1)));

// Aggregate demand for labor:
n = y-(1-kuu)*a-(kuu+alpha*(1-kuu))*wr+alpha*(1-kuu)*rk+kuu*mc;

// Aggregate demand for capital services:
k + u = y-(1-kuu)*a-(1-alpha*(1-kuu))*rk+(1-kuu)*(1-alpha)*wr+kuu*mc;

// Risk premium:
fii = -pessi*bystar+vi*zfiistar+zfii;

// Capital Euler equation:
qi = beta*(1-del)*qi(+1)+(1-beta*(1-del))*rk(+1)-(r-pi(+1));

// Investment Euler equation:
i = (1/(dels*(1+beta)))*qi+(beta/(1+beta))*i(+1)+(1/(1+beta))*i(-1)+((1-rhoi*beta)/(1+beta))*zi;

// Law of motion of capital:
k = (1-del)*k(-1)+iok*i;

// Export equation:
x = mstar+kappa*q;

// Import equation:
m = y-kuu*(q-mc);

// Capital utilization:
rk = dela*u;

// Real marginal cost:
mc = sd*(alpha*rk+(1-alpha)*wr-a)+(1-sd)*q;

// Phillips curve:
pi = ((1-tet*beta)*(1-omegabarb)*(1-tet)/(tet+omegabarb*(1-tet*(1-beta))))*mc
+(omegabarb/(tet+omegabarb*(1-tet*(1-beta))))*pi(-1)+(tet*beta/(tet+omegabarb*(1-tet*(1-beta))))*pi(+1);

// Law of motion of net foreign assets (as a proportion of GDP):
bystar = fiistar*rstarC*(bystar(-1)+nxy+bystarC*(yva(-1)-yva+(1/sva)*(q-q(-1))-pistar))+bystarC*(fii+rstar);  

// Net exports (as a proportion of GDP):
nxy = (sx/sva)*x-(sm/sva)*m-((sx-sm)/sva)*yva-(sm/sva)*((1-sx)/sva)*q;

// Primary surplus (as a proportion of GDP):
syhat + sgbar = -gy;

// Government debt (as a proportion of GDP):
bby = rC*(bby(-1)+gy-byC*(yva-yva(-1)+piva))+byC*r;

// Government expenditure:
g = yva+(sva/sg)*gy-(sm/sva)*q;

// Final good market equilibrium condition (resource constraint) - gross output:
y = sc*c+si*i+sg*g+sx*x;

// Value added:   
yva = (1/sva)*y-(sm/sva)*m;

// Value-added inflation (GDP de?ator):
piva = pi-(sm/sva)*(q-q(-1));

// Inflation target:
// pibar = rhopi*pibar(-1)+pibar_;
pibar = rhopi*pibar(-1);

// Fiscal target:
// sgbar = rhosbar*sgbar(-1)+gbar_;
sgbar = rhosbar*sgbar(-1);

// Household's preference:
zc = rhoc*zc(-1) + c_;

// Labor supply:
// zn = rhon*zn(-1) + n_;
zn = rhon*zn(-1);

// Investment Shock:
zi = rhoi*zi(-1) + i_;

// Foreign investor's risk aversion:
zfiistar = rhofiistar*zfiistar(-1) + fiistar_;

// Country risk premium shock:
// zfii = rhofii*zfii(-1) + fii_;
zfii = rhofii*zfii(-1);

// Technology:
a = rhoa*a(-1) + a_;

// Monetary policy:
zr = rhor*zr(-1) + r_;

// Fiscal policy:
// zg = rhog*zg(-1) + g_;
zg = rhog*zg(-1);

// World imports:
// mstar = rhomstar*mstar(-1) + mstar_;
mstar = rhomstar*mstar(-1);

// World inflation:
pistar = rhopistar*pistar(-1) + pistar_;

// World interest rate:
rstar = rhorstar*rstar(-1) + rstar_;
end;

//steady;

//**************************************************************************
//Declarao das variveis endogenas. - Note that they are the variables of the VAR (4 by default)
//Devem ter o mesmo tanto que variaveis exogenas

varobs 
y pi r q syhat rstar pistar;

//**************************************************************************
estimated_params;

sigma, normal_pdf, 1.5, 0.5;      
pessi, normal_pdf, 0.04, 0.02;         
kappa, gamma_pdf, 1, 0.25;    
h, beta_pdf, 0.5, 0.25;          
omegabarb, beta_pdf, 0.3, 0.08;    
tet, beta_pdf, 0.7, 0.15;          
nuu, gamma_pdf, 0.25, 0.2;          
kuu, gamma_pdf, 0.5, 0.2;      
vi, normal_pdf, 0.7, 0.5;           
dels, gamma_pdf, 2.5, 1.5;    
gamar, beta_pdf, 0.5, 0.25;        
gamapi, normal_pdf, 1.75, 0.15;     
gamay, normal_pdf, 0.25, 0.2;       
gamag, beta_pdf, 0.5, 0.25;         
gamas, normal_pdf, 0.5, 0.25;       
gamab, beta_pdf, 0.25, 0.1;          
rhoa, beta_pdf, 0.5, 0.25;        
rhoc, beta_pdf, 0.5, 0.25;          
rhoi, beta_pdf, 0.5, 0.25;         
rhon, beta_pdf, 0.5, 0.25;         
rhosbar, beta_pdf, 0.9, 0.08;     
rhopi, beta_pdf, 0.9, 0.08;      
rhofii, beta_pdf, 0.4, 0.2;      
rhofiistar, beta_pdf, 0.85, 0.1;   
rhomstar, beta_pdf, 0.25, 0.15;    
rhorstar, beta_pdf, 0.9, 0.05;  
rhopistar, normal_pdf, 0, 0.25;
// stderr pibar_, inv_gamma_pdf, 0.05, inf;
// stderr gbar_, inv_gamma_pdf, 0.05, inf;
stderr c_, inv_gamma_pdf, 1.2, inf;
stderr i_, inv_gamma_pdf, 2.5, inf;
// stderr n_, inv_gamma_pdf, 0.5, inf;
stderr a_, inv_gamma_pdf, 1, inf;
// stderr mstar_, inv_gamma_pdf, 0.8, inf;
stderr pistar_, inv_gamma_pdf, 0.3, inf;
stderr rstar_, inv_gamma_pdf, 0.01, inf;
stderr r_, inv_gamma_pdf, 1, inf;
// stderr g_, inv_gamma_pdf, 0.3, inf;
stderr fiistar_, inv_gamma_pdf, 0.25, inf;
// stderr fii_, inv_gamma_pdf, 0.5, inf;

// Calibrao para o peso dsge_prior - Peso ser estimado
// Deve estar no estimated_params block
dsge_prior_weight, uniform_pdf,,,0,2;

// Caso queira definir peso, comentar linha acima e descomentar a de baixo
// dsge_prior_weight, 1.75;
end;

//**************************************************************************
/* REMARK 1.
** The dsge_var option triggers the estimation of a DSGE-VAR model instead of the plain DSGE model. The weight of the DSGE prior, dsge_prior_weight,
** is estimated. The prior of this parameter is defined in the estimated_params block. SEM CALIBRACAO
**
** REMARK 2
** The option dsge_var= n triggers the estimation of a DSGE-VAR model, with a calibrated dsge prior weight equal to n.
**
** REMARK 3. 
** The option bayesian_irf triggers the computation of the DSGE-VAR and DSGE posterior distribution of the IRFs.
** The Dashed lines are the first, fifth (ie the median) and ninth posterior deciles of the DSGE-VAR's IRFs, the bold dark curve is the
** posterior median of the DSGE's IRfs and the shaded surface covers the space between the first and ninth posterior deciles of the DSGE's IRFs.
*/

estimation(datafile=dados2, forecast = 4, nobs = 54, plot_priors = 0, first_obs=20,mh_nblocks=2,mh_drop=0.45,mh_jscale=0.3,dsge_var,mode_compute=4,optim=('NumgradAlgorithm',3),mh_replic=2000,bayesian_irf);

/* Opcoes para Metropolis hasting.
** mh_jscale: Aceptance rate shoul be around 0.234 - Para alterar as taxas de aceitao, deve-se ajustar a opca mh_jscale
** mh_nblocks: number of MCMC sequences
*/
//**************************************************************************
/* SOLUO ESTOCSTICA E SOLUO.
** stoch_simul computes a Taylor approximation of the decision and transition functions for the model. 
** Using this, it computes impulse response functions and various descriptive statistics (moments, variance decomposition, correlation and autocorrelation coefficients). 
** For correlated shocks, the variance decomposition is computed as in the VAR literature through a Cholesky decomposition of the covariance matrix of the exogenous variables. 
** When the shocks are correlated, the variance decomposition depends upon the order of the variables in the varexo command.
*/
//stoch_simul(irf = 21, ar=100)pi r q syhat rstar pistar;

//**************************************************************************


//forecast (periods=4) pi r q syhat rstar pistar;

